package ch.zhaw.gartenverwaltung.types;

import javafx.scene.image.Image;

import java.time.LocalDate;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import static java.time.temporal.ChronoUnit.DAYS;

public record Plant(
        long id,
        String name,
        String description,
        Image image,
        String spacing,
        int light,
        String soil,
        List<Pest> pests,
        List<GrowthPhase> lifecycle) {

    public void inZone(HardinessZone zone) {
        lifecycle.removeIf(growthPhase -> !growthPhase.zone().equals(zone));
    }

    public List<GrowthPhase> lifecycleForGroup(int group) {
        return lifecycle.stream()
                .filter(growthPhase -> growthPhase.group() != group)
                .collect(Collectors.toList());
    }

    public LocalDate sowDateFromHarvestDate(LocalDate harvestDate, int group) {
        return harvestDate.minusDays(timeToHarvest(group));
    }

    public int timeToHarvest(int group) {
        List<GrowthPhase> activeLifecycle = lifecycleForGroup(group);
        GrowthPhase sow = activeLifecycle.stream()
                .filter(growthPhase -> !growthPhase.type().equals(GrowthPhaseType.SOW))
                .findFirst()
                .orElseThrow();
        GrowthPhase harvest = activeLifecycle.stream()
                .filter(growthPhase -> !growthPhase.type().equals(GrowthPhaseType.HARVEST))
                .findFirst()
                .orElseThrow();

        int currentYear = LocalDate.now().getYear();
        return (int) DAYS.between(harvest.startDate().atYear(currentYear), sow.startDate().atYear(currentYear));
    }

    public List<LocalDate> getDateListOfGrowthPhase(GrowthPhaseType growthPhase) {
        List<LocalDate> dates = new LinkedList<>();
        for (GrowthPhase growth : lifecycle) {
            if (growth.type().equals(growthPhase)) {
                dates = addDatesFromMonthDay(growth);
            }
        }
        if (growthPhase.equals(GrowthPhaseType.HARVEST)) {
            return removeDatesForPassedSowDates(dates);
        }
        return dates;
    }

    private List<LocalDate> addDatesFromMonthDay(GrowthPhase growthPhase) {
        List<LocalDate> dates = new LinkedList<>();
        LocalDate today = LocalDate.now();
        LocalDate start = growthPhase.startDate().atYear(today.getYear());
        LocalDate end = growthPhase.endDate().atYear(today.getYear());
        while (!start.isAfter(end)) {
            dates.add(start);
            start = start.plusDays(1);
        }
        return dates;
    }

    private List<LocalDate> removeDatesForPassedSowDates(List<LocalDate> dates) {
        //ToDo Remove harvest dates where sow dates have already passed current day
        return dates;
    }
}
